@@ -29,20 +29,32 @@ using System;
 using System.Collections;
 using System.IO;
 using System.Text;
+using System.Threading;
 using System.Xml;
 using System.Xml.Serialization;
 using Beagle.Util;
 
 namespace Beagle {
 
-	public enum IndexableFiltering {
-		Never,     // Never try to filter this indexable
-		Automatic, // Try to determine automatically if this needs to be filtered
-		Always     // Always try to filter this indexable
+	public enum IndexableType {
+		Add,
+		Remove,
+		PropertyChange
 	}
 
-	public class Indexable : Versioned {
+	public enum IndexableFiltering {
+		Never,           // Never try to filter this indexable, it contains no content
+		AlreadyFiltered, // The readers promise to return nice clean text, so do nothing
+		Automatic,       // Try to determine automatically if this needs to be filtered
+		Always           // Always try to filter this indexable
+	}
 
+	public class Indexable : Versioned, IComparable {
+
+		// This is the type of indexing operation represented by
+		// this Indexable object.  We default to Add, for historical
+		// reasons.
+		private IndexableType type = IndexableType.Add;
 
 		// The URI of the item being indexed.
 		private Uri uri = null;
@@ -60,7 +72,7 @@ namespace Beagle {
 		private bool deleteContent = false;
 		
 		// File, WebLink, MailMessage, IMLog, etc.
-		private String type = null;
+		private String hit_type = null;
 
 		// If applicable, otherwise set to null.
 		private String mimeType = null;
@@ -91,6 +103,10 @@ namespace Beagle {
 		// When should we try to filter this indexable?
 		private IndexableFiltering filtering = IndexableFiltering.Automatic;
 
+		// Local state: these are key/value pairs that never get serialized
+		// into XML
+		Hashtable local_state = new Hashtable ();
+
 		//////////////////////////
 
 		static private XmlSerializer our_serializer;
@@ -102,13 +118,19 @@ namespace Beagle {
 
 		//////////////////////////
 
-		public Indexable (Uri _uri) {
-			uri = _uri;
-
-			type = "File";
+		public Indexable (IndexableType type,
+				  Uri           uri)
+		{
+			this.type = type;
+			this.uri = uri;
+			this.hit_type = "File"; // FIXME: Why do we default to this?
 		}
 
-		public Indexable () {
+		public Indexable (Uri uri) : this (IndexableType.Add, uri)
+		{ }
+
+		public Indexable ()
+		{
 			// Only used when reading from xml
 		}
 
@@ -120,6 +142,12 @@ namespace Beagle {
 
 		//////////////////////////
 
+		[XmlAttribute ("Type")]
+		public IndexableType Type {
+			get { return type; }
+			set { type = value; }
+		}
+
 		[XmlIgnore]
 		public Uri Uri { 
 			get { return uri; }
@@ -181,7 +209,7 @@ namespace Beagle {
 
 		[XmlIgnore]
 		public Uri DisplayUri {
-			get { return uri.Scheme == "uid" ? ContentUri : Uri; }
+			get { return uri.Scheme == GuidFu.UriScheme ? ContentUri : Uri; }
 		}
 
 		[XmlAttribute]
@@ -191,9 +219,9 @@ namespace Beagle {
 		}
 
 		[XmlAttribute]
-		public String Type {
-			get { return type; }
-			set { type = value; }
+		public String HitType {
+			get { return  hit_type; }
+			set { hit_type = value; }
 		}
 
 		[XmlAttribute]
@@ -231,8 +259,39 @@ namespace Beagle {
 			set { filtering = value; }
 		}
 
+		[XmlIgnore]
+		public IDictionary LocalState {
+			get { return local_state; }
+		}
+
 		//////////////////////////
 
+		public void Cleanup ()
+		{
+			if (DeleteContent) {
+				if (contentUri != null) {
+					//Logger.Log.Debug ("Cleaning up {0}", contentUri.LocalPath);
+					try {
+						File.Delete (contentUri.LocalPath);
+					} catch (Exception ex)
+					{ 
+						// It might be gone already, so catch the exception.
+					}
+					contentUri = null;
+				}
+				if (hotContentUri != null) {
+					//Logger.Log.Debug ("Cleaning up {0}", hotContentUri.LocalPath);
+					try {
+						File.Delete (hotContentUri.LocalPath);
+					} catch (Exception ex)
+					{
+						// Ditto
+					}
+					hotContentUri = null;
+				}
+			}
+		}
+
 		private Stream StreamFromUri (Uri uri)
 		{
 			Stream stream = null;
@@ -242,10 +301,6 @@ namespace Beagle {
 							 FileMode.Open,
 							 FileAccess.Read,
 							 FileShare.Read);
-
-				// Paranoia: never delete the thing we are actually indexing.
-				if (DeleteContent && uri != Uri)
-					File.Delete (uri.LocalPath);
 			}
 
 			return stream;
@@ -315,7 +370,26 @@ namespace Beagle {
 		}
 
 		public void AddProperty (Property prop) {
-			properties.Add (prop);
+			if (prop != null) {
+				
+				if (type == IndexableType.PropertyChange && ! prop.IsMutable)
+					throw new ArgumentException ("Non-mutable properties aren't allowed in this indexable");
+
+				// If this is a mutable property, make sure that
+				// we don't already contain another mutable property
+				// with the same name.  If we do, replace it.
+				if (prop.IsMutable) {
+					for (int i = 0; i < properties.Count; ++i) {
+						Property other_prop = properties [i] as Property;
+						if (other_prop.IsMutable && prop.Key == other_prop.Key) {
+							properties [i] = prop;
+							return;
+						}
+					}
+				}
+
+				properties.Add (prop);
+			}
 		}
 
 		public bool HasProperty (string keyword) {
@@ -326,30 +400,19 @@ namespace Beagle {
 			return false;
 		}
 
-		//////////////////////////
-
-		private string PropertiesAsString (bool keywords)
+		// This doesn't check if it makes sense to actually
+		// merge the two indexables: it just does it.
+		public void Merge (Indexable other)
 		{
-			StringBuilder sb = new StringBuilder ();
-			foreach (Property prop in Properties) {
-				if (prop.IsSearched 
-				    && keywords ? prop.IsKeyword : ! prop.IsKeyword) {
-					if (sb.Length > 0)
-						sb.Append (" ");
-					sb.Append (prop.Value);
-				}
-			}
-			return sb.ToString ();
+			this.Timestamp = other.Timestamp;
+
+			foreach (Property prop in other.Properties)
+				this.AddProperty (prop);
+
+			foreach (DictionaryEntry entry in other.local_state)
+				this.local_state [entry.Key] = entry.Value;
 		}
 
-		public string TextPropertiesAsString {
-			get { return PropertiesAsString (false); }
-		}
-
-		public string KeywordPropertiesAsString {
-			get { return PropertiesAsString (true); }
-		}
-		
 		//////////////////////////
 
 		public void SetChildOf (Indexable parent)
@@ -380,6 +443,48 @@ namespace Beagle {
 			return writer.ToString ();
 		}
 
+		//////////////////////////
+
+		const int BUFFER_SIZE = 8192;
+
+		private static char [] GetCharBuffer ()
+		{
+			LocalDataStoreSlot slot;
+			slot = Thread.GetNamedDataSlot ("Char Buffer");
+
+			object obj;
+			char [] buffer;
+			obj = Thread.GetData (slot);
+			if (obj == null) {
+				buffer = new char [BUFFER_SIZE];
+				Thread.SetData (slot, buffer);
+			} else {
+				buffer = (char []) obj; 
+			}
+
+			return buffer;
+		}
+
+		private static byte [] GetByteBuffer ()
+		{
+			LocalDataStoreSlot slot;
+			slot = Thread.GetNamedDataSlot ("Byte Buffer");
+
+			object obj;
+			byte [] buffer;
+			obj = Thread.GetData (slot);
+			if (obj == null) {
+				buffer = new byte [BUFFER_SIZE];
+				Thread.SetData (slot, buffer);
+			} else {
+				buffer = (byte []) obj; 
+			}
+
+			return buffer;
+		}
+
+		//////////////////////////
+
 		private static Uri TextReaderToTempFileUri (TextReader reader)
 		{
 			if (reader == null)
@@ -395,17 +500,18 @@ namespace Beagle {
 			// Make sure the temporary file is only readable by the owner.
 			// FIXME: There is probably a race here.  Could some malicious program
 			// do something to the file between creation and the chmod?
-			Mono.Posix.Syscall.chmod (filename, (Mono.Posix.FileMode) 256);
+			Mono.Unix.Syscall.chmod (filename, (Mono.Unix.FilePermissions) 256);
 
 			BufferedStream bufferedStream = new BufferedStream (fileStream);
 			StreamWriter writer = new StreamWriter (bufferedStream);
 
-			const int BUFFER_SIZE = 8192;
-			char [] buffer = new char [BUFFER_SIZE];
+
+			char [] buffer;
+			buffer = GetCharBuffer ();
 
 			int read;
 			do {
-				read = reader.Read (buffer, 0, BUFFER_SIZE);
+				read = reader.Read (buffer, 0, buffer.Length);
 				if (read > 0)
 					writer.Write (buffer, 0, read);
 			} while (read > 0);
@@ -430,16 +536,16 @@ namespace Beagle {
 			// Make sure the temporary file is only readable by the owner.
 			// FIXME: There is probably a race here.  Could some malicious program
 			// do something to the file between creation and the chmod?
-			Mono.Posix.Syscall.chmod (filename, (Mono.Posix.FileMode) 256);
+			Mono.Unix.Syscall.chmod (filename, (Mono.Unix.FilePermissions) 256);
 
 			BufferedStream bufferedStream = new BufferedStream (fileStream);
 
-			const int BUFFER_SIZE = 8192;
-			byte [] buffer = new byte [BUFFER_SIZE];
+			byte [] buffer;
+			buffer = GetByteBuffer ();
 
 			int read;
 			do {
-				read = stream.Read (buffer, 0, BUFFER_SIZE);
+				read = stream.Read (buffer, 0, buffer.Length);
 				if (read > 0)
 					bufferedStream.Write (buffer, 0, read);
 			} while (read > 0);
@@ -452,14 +558,17 @@ namespace Beagle {
 		public void StoreStream () {
 			if (textReader != null) {
 				ContentUri = TextReaderToTempFileUri (textReader);
+				//Logger.Log.Debug ("Storing text content from {0} in {1}", Uri, ContentUri);
 				DeleteContent = true;
 			} else if (binary_stream != null) {
 				ContentUri = BinaryStreamToTempFileUri (binary_stream);
+				//Logger.Log.Debug ("Storing binary content from {0} in {1}", Uri, ContentUri);
 				DeleteContent = true;
 			}
 
 			if (hotTextReader != null) {
 				HotContentUri = TextReaderToTempFileUri (hotTextReader);
+				//Logger.Log.Debug ("Storing hot content from {0} in {1}", Uri, HotContentUri);
 				DeleteContent = true;
 			}
 		}
@@ -470,5 +579,11 @@ namespace Beagle {
 		{
 			return (uri != null ? uri.GetHashCode () : 0) ^ type.GetHashCode ();
 		}
+
+		public int CompareTo (object obj)
+		{
+			Indexable other = (Indexable) obj;
+			return DateTime.Compare (this.Timestamp, other.Timestamp);
+		}
 	}
 }
