@@ -7,12 +7,13 @@
 
 using System;
 using System.Collections;
+using System.Diagnostics;
 using System.Runtime.InteropServices;
 
 using Gtk;
 using Gdk;
 
-using Mono.Posix;
+using Mono.Unix;
 
 using Beagle;
 using Beagle.Tile;
@@ -65,16 +66,19 @@ namespace Best {
 	public class BestTray : Gtk.Plug
 	{
 		BestWindow win;
+		bool autostarted = false;
 		
 		Gtk.EventBox eventbox;
 		Gtk.Tooltips tips;
 		Beagle.Util.XKeybinder keybinder;
 
-		[DllImport ("libtrayiconglue")]
+		[DllImport ("libbeagleuiglue")]
 		private static extern IntPtr egg_tray_icon_new (string name);
 
-		public BestTray (BestWindow bw)
+		public BestTray (BestWindow bw, bool autostarted)
 		{
+			this.autostarted = autostarted;
+
 			Raw = egg_tray_icon_new ("Search");
 
 			win = bw;
@@ -137,12 +141,39 @@ namespace Best {
 
 		void QuitEvent (object sender, EventArgs args)
 		{
+			if (autostarted) {
+				HigMessageDialog dialog = new HigMessageDialog (win,
+										DialogFlags.Modal,
+										MessageType.Question,
+										ButtonsType.YesNo,
+										Catalog.GetString ("Disable Searching"), 
+										Catalog.GetString ("You are about to close the search tray. The search tray is automatically started at login-time. Would you like to disable it for future sessions?"));
+
+				Gtk.ResponseType response = (Gtk.ResponseType) dialog.Run ();
+				
+				if (response == Gtk.ResponseType.Yes) {
+					Conf.Searching.Autostart = false;
+					Conf.Save (true);
+					
+					// If the user doesn't want to have Best autostart, he probably doesn't 
+					// want to keep the daemon around either. Bad call dude, bad call.
+					
+					Process p = new Process ();
+					p.StartInfo.UseShellExecute = false;
+					p.StartInfo.FileName = "beagle-shutdown";
+
+					try {
+						p.Start ();
+					} catch (Exception ex) {}
+				}
+			}
+			win.StoreSettingsInConf ();
 			Application.Quit ();
 		}
 		
 		void QuickSearchEvent (object sender, EventArgs args) 
-		{			
-			string quickQuery = (string)((Gtk.Widget) sender).Data ["Query"];
+		{		
+			string quickQuery = (string) menu_to_query_map [sender];
 						
 			if (! win.WindowIsVisible) {
 				win.Present ();
@@ -157,11 +188,17 @@ namespace Best {
 		{			
 			win.ClearHistory ();
 		}
-		
+
+		private void DetachWidget (Gtk.Widget attach_widget, Gtk.Menu menu)
+		{
+		}
+
+		Hashtable menu_to_query_map = null;
+
 		private Gtk.Menu MakeMenu (Gtk.Widget parent) 
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
-			menu.AttachToWidget (parent, null);
+			menu.AttachToWidget (parent, new Gtk.MenuDetachFunc (DetachWidget));
 			
 			Gtk.ImageMenuItem item;
 						
@@ -171,17 +208,20 @@ namespace Best {
 				item = new Gtk.ImageMenuItem (Catalog.GetString ("No Recent Searches"));
 				item.Sensitive = false;
 				menu.Append (item);
+				menu_to_query_map = null;
 			} else {
 				item = new Gtk.ImageMenuItem (Catalog.GetString ("Recent Searches"));
 				item.Sensitive = false;
 				item.Image = new Gtk.Image (Images.GetPixbuf ("icon-search.png"));
 				menu.Append (item);
 
+				menu_to_query_map = new Hashtable ();
+
 				foreach (string s in list) {
 					item = new Gtk.ImageMenuItem (s);
-					item.Data ["Query"] = s;
 					item.Activated += new EventHandler (QuickSearchEvent);
 					menu.Append (item);
+					menu_to_query_map [item] = s;
 				}
 			}			
 
