@@ -30,7 +30,9 @@ using System.Collections;
 using System.IO;
 using System.Text;
 using System.Xml;
+
 using Beagle.Util;
+using Beagle.Daemon;
 
 using ICSharpCode.SharpZipLib.Zip;
 
@@ -44,20 +46,24 @@ namespace Beagle.Filters {
 		public FilterOpenOffice () 
 		{
 			// OO 1.0 mime types
-			AddSupportedMimeType ("application/vnd.sun.xml.writer");
-			AddSupportedMimeType ("application/vnd.sun.xml.writer.template");
-			AddSupportedMimeType ("application/vnd.sun.xml.calc");
-			AddSupportedMimeType ("application/vnd.sun.xml.calc.template");
-			AddSupportedMimeType ("application/vnd.sun.xml.impress");
-			AddSupportedMimeType ("application/vnd.sun.xml.impress.template");
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.writer"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.writer.template"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.calc"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.calc.template"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.impress"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.impress.template"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.draw"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.sun.xml.draw.template"));
 
 			// OO 2.0 mime types
-			AddSupportedMimeType ("application/vnd.oasis.opendocument.text");
-			AddSupportedMimeType ("application/vnd.oasis.opendocument.text-template");
-			AddSupportedMimeType ("application/vnd.oasis.opendocument.spreadsheet");
-			AddSupportedMimeType ("application/vnd.oasis.opendocument.spreadsheet-template");
-			AddSupportedMimeType ("application/vnd.oasis.opendocument.presentation");
-			AddSupportedMimeType ("application/vnd.oasis.opendocument.presentation-template");
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.text"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.text-template"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.spreadsheet"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.spreadsheet-template"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.presentation"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.presentation-template"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.graphics"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.oasis.opendocument.graphics-template"));
 
 			SnippetMode = true;
 			odtFormat = false;
@@ -340,7 +346,8 @@ namespace Beagle.Filters {
 					if (hot_nodes.Count > 0)
 						is_hot = (bool) hot_nodes.Pop ();
 					else
-						Logger.Log.Debug ("FilterOpenOffice: hot_nodes underflow in {0}", reader.Name);
+						Logger.Log.Debug ("FilterOpenOffice: hot_nodes underflow in {0}", 
+								  reader.Name);
 					if (hot_container_nodes.Count > 0) {
 						string hot_container_tag = (string) hot_container_nodes.Peek ();
 						if (hot_container_tag == reader.Name) {
@@ -477,7 +484,8 @@ namespace Beagle.Filters {
 					odtFormat = true;
 
 			} catch (Exception e) {
-				Logger.Log.Error ("Unable to open {0}.  Probably an invalid OpenOffice document.");
+				Logger.Log.Error ("Unable to open {0}.  Probably an invalid OpenOffice document.", 
+						  info.FullName);
 				Finished ();
 			}
 		}
