@@ -29,9 +29,9 @@ using System.Collections;
 using System.Diagnostics;
 using System.Text;
 using System.Text.RegularExpressions;
-using BU = Beagle.Util;
+using Beagle.Util;
 using GMime;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Beagle.Tile {
 
@@ -45,12 +45,19 @@ namespace Beagle.Tile {
 		{
 		}
 
+		private static bool IsAttachment (Hit hit)
+		{
+			// check if there is parent and parent has attachments
+			string str = hit ["parent:fixme:hasAttachments"];
+			return (hit.ParentUri != null && str != null && (str == "true"));
+		}
+		
 		private static string GetHitProperty (Hit hit, string name)
 		{
 			// FIXME: We should handle this case better, but
 			// for now, if we match an attachment, we just want
 			// to display the properties for the parent message.
-			if (hit.ParentUri == null)
+			if (!IsAttachment (hit))
 				return hit [name];
 			else
 				return hit ["parent:" + name];
@@ -120,14 +127,13 @@ namespace Beagle.Tile {
 				Template ["CanReply"] = "";
 
 			// FIXME: Gross attachment rendering
-			if (Hit.ParentUri != null) {
+			if (IsAttachment (Hit)) {
 				Template["Subject"] = Hit ["fixme:attachment_title"] + " [" + Catalog.GetString ("Email attachment") + "]";
 				Template["EmailSubject"] = str;
-				Gtk.IconSize size = (Gtk.IconSize) 48;
-				string path = BU.GnomeIconLookup.LookupMimeIcon (Hit.MimeType, size);
-				Template["Icon"] = Images.GetHtmlSource (path, BU.GnomeIconLookup.GetFileMimeType (path));
+				string path = GnomeFu.GetMimeIconPath (Hit.MimeType);
+				Template["Icon"] = Images.GetHtmlSource (path, GnomeFu.GetMimeType (path));
 			}
-			
+
 			if (GetHitProperty (Hit, "fixme:isFlagged") != null)
 				Template["FollowupIcon"] = Images.GetHtmlSourceForStock ("stock_mail-priority-high", 16);
 			if (GetHitProperty (Hit, "fixme:hasAttachments") != null)
@@ -143,7 +149,7 @@ namespace Beagle.Tile {
 #if ENABLE_GALAGO
 #if ENABLE_EVO_SHARP
 			if (aim_name != null) {
-				string status = BU.GalagoTools.GetPresence ("aim", aim_name);
+				string status = GalagoTools.GetPresence ("aim", aim_name);
 				if (status != null && status != "")
 					Template ["Presence"] = status;
 			}
