@@ -30,7 +30,6 @@ using System;
 using System.IO;
 using System.Text;
 using System.Runtime.InteropServices;
-using Mono.Posix;
 
 using Beagle.Util;
 
@@ -93,14 +92,9 @@ namespace Beagle.Util {
 				FileStream file = new FileStream ("/etc/fstab",
 								  System.IO.FileMode.Open,
 								  FileAccess.Read);
-
 				int ret = GiveAdvice (file, AdviseNormal);
-				if (ret != 0) {
-					Logger log = Logger.Get ("FileAdvise");
-					log.Error ("FileAdvise failed: " +
-						   Syscall.strerror (Marshal.GetLastWin32Error()));
-				}
-
+				if (ret != 0)
+					Log.Error ("FileAdvise failed: {0}", Mono.Unix.Stdlib.strerror (Mono.Unix.Syscall.GetLastError()));
 				file.Close ();
 			} catch { }
 		}
