@@ -34,14 +34,16 @@ using System.Text;
 using System.Text.RegularExpressions;
 using System.Threading;
 
-using Mono.Posix;
-
 namespace Beagle.Util {
 
 	public class Inotify {
 
 		public delegate void InotifyCallback (Watch watch, string path, string subitem, string srcpath, EventType type);
 
+		public interface Watch {
+			void Unsubscribe ();
+			void ChangeSubscription (EventType new_mask);
+		}
 		/////////////////////////////////////////////////////////////////////////////////////
 
 		[Flags]
@@ -93,25 +95,81 @@ namespace Beagle.Util {
 			public uint      len;
 		}
 
-		[DllImport ("libinotifyglue")]
+		[DllImport ("libbeagleglue")]
 		static extern int inotify_glue_init ();
 
-		[DllImport ("libinotifyglue")]
+		[DllImport ("libbeagleglue")]
 		static extern int inotify_glue_watch (int fd, string filename, EventType mask);
 
-		[DllImport ("libinotifyglue")]
+		[DllImport ("libbeagleglue")]
 		static extern int inotify_glue_ignore (int fd, int wd);
 
-		[DllImport ("libinotifyglue")]
+		[DllImport ("libbeagleglue")]
 		static extern unsafe void inotify_snarf_events (int fd,
 								out int nr,
 								out IntPtr buffer);
 
-		[DllImport ("libinotifyglue")]
+		[DllImport ("libbeagleglue")]
 		static extern void inotify_snarf_cancel ();
 
 		/////////////////////////////////////////////////////////////////////////////////////
 
+		static public bool Verbose = false;
+		static private int inotify_fd = -1;
+
+		static Inotify ()
+		{
+			log = Logger.Get ("Inotify");
+
+			if (Environment.GetEnvironmentVariable ("BEAGLE_DISABLE_INOTIFY") != null) {
+				Logger.Log.Debug ("BEAGLE_DISABLE_INOTIFY is set");
+				return;
+			}
+
+			if (Environment.GetEnvironmentVariable ("BEAGLE_INOTIFY_VERBOSE") != null)
+				Inotify.Verbose = true;
+
+			try {
+				inotify_fd = inotify_glue_init ();
+			} catch (EntryPointNotFoundException) {
+				Logger.Log.Info ("Inotify not available on system.");
+				return;
+			}
+
+			if (inotify_fd == -1)
+				Logger.Log.Warn ("Could not initialize inotify");
+		}
+
+		static public bool Enabled {
+			get { return inotify_fd >= 0; }
+		}
+
+		/////////////////////////////////////////////////////////////////////////////////////
+
+#if ! ENABLE_INOTIFY
+
+		// Stubs for systems where inotify is unavailable
+
+		static public Watch Subscribe (string path, InotifyCallback callback, EventType mask)
+		{
+			return null;
+		}
+
+		static public void Start ()
+		{
+			return;
+		}
+
+		static public void Stop ()
+		{
+			return;
+		}
+
+#else // ENABLE_INOTIFY
+
+		/////////////////////////////////////////////////////////////////////////////////////
+		static private ArrayList event_queue = new ArrayList ();
+
 		private class QueuedEvent {
 			public int       Wd;
 			public EventType Type;
@@ -150,37 +208,6 @@ namespace Beagle.Util {
 
 		/////////////////////////////////////////////////////////////////////////////////////
 
-		static private int inotify_fd = -1;
-		static private ArrayList event_queue = new ArrayList ();
-
-		static Inotify ()
-		{
-			log = Logger.Get ("Inotify");
-
-			if (Environment.GetEnvironmentVariable ("BEAGLE_DISABLE_INOTIFY") != null) {
-				Logger.Log.Debug ("BEAGLE_DISABLE_INOTIFY is set");
-				return;
-			}
-
-			if (Environment.GetEnvironmentVariable ("BEAGLE_INOTIFY_VERBOSE") != null)
-				Inotify.Verbose = true;
-
-			inotify_fd = inotify_glue_init ();
-			if (inotify_fd == -1)
-				Logger.Log.Warn ("Could not initialize inotify");
-		}
-
-		static public bool Enabled {
-			get { return inotify_fd >= 0; }
-		}
-
-		/////////////////////////////////////////////////////////////////////////////////////
-
-		public interface Watch {
-			void Unsubscribe ();
-			void ChangeSubscription (EventType new_mask);
-		}
-
 		private class WatchInternal : Watch {
 			private InotifyCallback callback;
 			private EventType mask;
@@ -440,11 +467,8 @@ namespace Beagle.Util {
 
 				running = true;
 
-				snarf_thread = new Thread (new ThreadStart (SnarfWorker));
-				snarf_thread.Start ();
-
-				dispatch_thread = new Thread (new ThreadStart (DispatchWorker));
-				dispatch_thread.Start ();
+				snarf_thread = ExceptionHandlingThread.Start (new ThreadStart (SnarfWorker));
+				dispatch_thread = ExceptionHandlingThread.Start (new ThreadStart (DispatchWorker));
 			}
 		}
 
@@ -532,8 +556,6 @@ namespace Beagle.Util {
 		}
 
 
-		static public bool Verbose = false;
-
 		// Update the watched_by_path hash and the path stored inside the watch
 		// in response to a move event.
 		static private void MoveWatch (WatchInfo watch, string name)		
@@ -824,5 +846,7 @@ namespace Beagle.Util {
 		}
 #endif
 
+#endif // ENABLE_INOTIFY
+
 	}
 }
