@@ -36,7 +36,6 @@ using Beagle.Util;
 using Beagle.Daemon;
 
 using Camel = Beagle.Util.Camel;
-using Mono.Posix;
 
 namespace Beagle.Daemon.EvolutionMailDriver {
 
@@ -108,7 +107,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		protected void CrawlFinished ()
 		{
 			// FIXME: This is a little sketchy
-			this.queryable.FileAttributesStore.AttachLastWriteTime (this.CrawlFile.FullName, DateTime.Now);
+			this.queryable.FileAttributesStore.AttachLastWriteTime (this.CrawlFile.FullName, DateTime.UtcNow);
 		}
 
 		public string StatusName {
@@ -214,7 +213,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					return false;
 				}
 
-				this.mbox_fd = Syscall.open (this.mbox_info.FullName, OpenFlags.O_RDONLY);
+				this.mbox_fd = Mono.Unix.Syscall.open (this.mbox_info.FullName, Mono.Unix.OpenFlags.O_RDONLY);
 				this.mbox_stream = new GMime.StreamFs (this.mbox_fd);
 				this.mbox_stream.Seek ((int) this.MboxLastOffset);
 				this.mbox_parser = new GMime.Parser (this.mbox_stream);
@@ -301,7 +300,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			System.Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, uid);
 			Indexable indexable = new Indexable (uri);
 
-			indexable.Timestamp = message.Date;
+			indexable.Timestamp = message.Date.ToUniversalTime ();
 			indexable.HitType = "MailMessage";
 			indexable.MimeType = "message/rfc822";
 			indexable.CacheContent = false;
