@@ -31,8 +31,6 @@ using System.IO;
 using SNS = System.Net.Sockets;
 using System.Threading;
 
-using Mono.Posix;
-
 using Gtk;
 
 using Beagle.Daemon;
@@ -46,15 +44,31 @@ namespace Beagle.IndexHelper {
 		static Server server;
 
 		static void Main (string [] args)
+		{
+			try {
+				DoMain (args);
+			} catch (Exception ex) {
+				Logger.Log.Error ("Unhandled exception thrown.  Exiting immediately.");
+				Logger.Log.Error (ex);
+				Environment.Exit (1);
+			}
+		}
+
+		static void DoMain (string [] args)
 		{
 			bool run_by_hand = (Environment.GetEnvironmentVariable ("BEAGLE_RUN_HELPER_BY_HAND") != null);
 			bool log_in_fg = (Environment.GetEnvironmentVariable ("BEAGLE_LOG_IN_THE_FOREGROUND_PLEASE") != null);
+			bool debug = (Environment.GetEnvironmentVariable ("BEAGLE_DEBUG_FLAG_IS_SET") != null);
 
 			last_activity = DateTime.Now;
 
-			Logger.DefaultLevel = LogLevel.Debug;
-
-			Logger.LogToFile (PathFinder.LogDir, "IndexHelper", run_by_hand || log_in_fg);
+			Log.Initialize (PathFinder.LogDir,
+					"IndexHelper",
+					// FIXME: We always turn on full debugging output!  We are still
+					// debugging this code, after all...
+					//debug ? LogLevel.Debug : LogLevel.Warn,
+					LogLevel.Debug,
+					run_by_hand || log_in_fg);
 
 			Application.InitCheck ("IndexHelper", ref args);
 
@@ -96,8 +110,6 @@ namespace Beagle.IndexHelper {
 					} catch (IOException) { }
 				}
 			}
-
-			Environment.Exit (0);
 		}
 
 		static public void ReportActivity ()
@@ -167,7 +179,7 @@ namespace Beagle.IndexHelper {
 			try {
 				SNS.Socket socket;
 				socket = new SNS.Socket (SNS.AddressFamily.Unix, SNS.SocketType.Stream, 0);
-				socket.Connect (new UnixEndPoint (socket_name));
+				socket.Connect (new Mono.Unix.UnixEndPoint (socket_name));
 				
 				ArrayList socket_list = new ArrayList ();
 				
@@ -189,25 +201,16 @@ namespace Beagle.IndexHelper {
 
 		/////////////////////////////////////////////////////////////////////////////
 
-		// The integer values of the Mono.Posix.Signal enumeration don't actually
-		// match the Linux signal numbers of Linux.  Oops!
-		// This is fixed in Mono.Unix, but for the moment we want to maintain
-		// compatibility with mono 1.0.x.
-		const int ACTUAL_LINUX_SIGINT  = 2;
-		const int ACTUAL_LINUX_SIGQUIT = 3;
-		const int ACTUAL_LINUX_SIGTERM = 15;
-
 		static void SetupSignalHandlers ()
 		{
 			// Force OurSignalHandler to be JITed
 			OurSignalHandler (-1);
 
 			// Set up our signal handler
-			Mono.Posix.Syscall.sighandler_t sig_handler;
-			sig_handler = new Mono.Posix.Syscall.sighandler_t (OurSignalHandler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGINT, sig_handler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGQUIT, sig_handler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGTERM, sig_handler);
+			Mono.Unix.Stdlib.signal (Mono.Unix.Signum.SIGINT, OurSignalHandler);
+			Mono.Unix.Stdlib.signal (Mono.Unix.Signum.SIGTERM, OurSignalHandler);
+			if (Environment.GetEnvironmentVariable("BEAGLE_THERE_BE_NO_QUITTIN") == null)
+				Mono.Unix.Stdlib.signal (Mono.Unix.Signum.SIGQUIT, OurSignalHandler);
 		}
 
 		// Our handler triggers an orderly shutdown when it receives a signal.
@@ -254,7 +257,8 @@ namespace Beagle.IndexHelper {
 
 		static void OnShutdown ()
 		{
-			server.Stop ();
+			if (server != null)
+				server.Stop ();
 		}
 	}
 
