@@ -33,13 +33,8 @@ using Beagle;
 
 namespace Beagle.Daemon {
 
-	public delegate void QueryableChangedHandler (Queryable source,
-						      IQueryableChangeData changeData);
-
 	public class Queryable {
 
-		public event QueryableChangedHandler ChangedEvent;
-
 		private QueryableFlavor flavor;
 		private IQueryable iqueryable;
 
@@ -48,8 +43,6 @@ namespace Beagle.Daemon {
 		{
 			flavor = _flavor;
 			iqueryable = _iqueryable;
-
-			iqueryable.ChangedEvent += OnIChangedEvent;
 		}
 		
 		public void Start ()
@@ -64,79 +57,57 @@ namespace Beagle.Daemon {
 		public QueryDomain Domain {
 			get { return flavor.Domain; }
 		}
-			
-		public bool AcceptQuery (QueryBody body)
+
+		public bool AcceptQuery (Query query)
 		{
-			return body != null
-				&& ! body.IsEmpty
-				&& body.AllowsSource (Name)
-				&& body.AllowsDomain (Domain)
-				&& iqueryable.AcceptQuery (body);
+			return query != null
+				&& ! query.IsEmpty
+				&& query.AllowsSource (Name)
+				&& query.AllowsDomain (Domain)
+				&& iqueryable.AcceptQuery (query);
 		}
 
-		public string GetHumanReadableStatus ()
+		public void DoQuery (Query query, IQueryResult result, IQueryableChangeData change_data)
 		{
-			return iqueryable.GetHumanReadableStatus ();
-		}
-
-		public int GetItemCount ()
-		{
-			int n = -1;
 			try {
-				n = iqueryable.GetItemCount ();
+				iqueryable.DoQuery (query, result, change_data);
 			} catch (Exception ex) {
-				Logger.Log.Warn ("Couldn't get item count for '{0}'", Name);
+				Logger.Log.Warn ("Caught exception calling DoQuery on '{0}'", Name);
 				Logger.Log.Warn (ex);
 			}
-			return n;
 		}
 
-		//////////////////////////////////////////////////////////////
-
-		private void OnIChangedEvent (IQueryable source, IQueryableChangeData changeData)
+		public string GetSnippet (string[] query_terms, Hit hit)
 		{
-			if (source != iqueryable)
-				return;
-			if (ChangedEvent != null)
-				ChangedEvent (this, changeData);
-		}
+			if (hit == null)
+				return null;
 
-		//////////////////////////////////////////////////////////////
+			// Sanity-check: make sure this Hit actually came out of this Queryable
+			if (QueryDriver.GetQueryable (hit.SourceObjectName) != this) {
+				string msg = String.Format ("Queryable mismatch in GetSnippet: {0} vs {1}", hit.SourceObjectName, this);
+				throw new Exception (msg);
+			}
 
-		private class QueryClosure : IQueryWorker{
-
-			private Queryable queryable;
-			private QueryBody body;
-			private IQueryableChangeData changeData;
+			try {
+				return iqueryable.GetSnippet (query_terms, hit);
+			} catch (Exception ex) {
+				Logger.Log.Warn ("Caught exception calling DoQuery on '{0}'", Name);
+				Logger.Log.Warn (ex);
+			}
 			
-			public QueryClosure (Queryable            _queryable,
-					     QueryBody            _body,
-					     IQueryableChangeData _changeData)
-			{
-				queryable  = _queryable;
-				body       = _body;
-				changeData = _changeData;
-			}
-
-			public void PerformWork (QueryResult result)
-			{
-				queryable.iqueryable.DoQuery (body, result, changeData);
-			}
-
-			public override string ToString ()
-			{
-				return "QueryWorker: " + queryable.iqueryable.ToString ();
-			}
+			return null;
 		}
 
-
-		public void DoQuery (QueryBody body,
-				     QueryResult result,
-				     IQueryableChangeData data)
+		public QueryableStatus GetQueryableStatus ()
 		{
-			QueryClosure qc;
-			qc = new QueryClosure (this, body, data);
-			result.AttachWorker (qc);
+		        QueryableStatus status = iqueryable.GetQueryableStatus ();
+
+			if (status == null)
+				status = new QueryableStatus ();
+
+			status.Name = Name;
+
+			return status;
 		}
 	}
 }
